from __future__ import print_function
import httplib2
import os
from google_auth_oauthlib.flow import InstalledAppFlow
from apiclient import discovery
from oauth2client import client
from oauth2client import tools
from oauth2client.file import Storage
from google.auth.transport.requests import Request
import pickle
try:
    import argparse
    flags = argparse.ArgumentParser(parents=[tools.argparser]).parse_args()
except ImportError:
    flags = None


class auth:
    def __init__(self,SCOPES ,
                CLIENT_SECRET_FILE =None ,
                APPLICATION_NAME = None):
        self.SCOPES = SCOPES
        self.CLIENT_SECRET_FILE = CLIENT_SECRET_FILE
        self.APPLICATION_NAME = APPLICATION_NAME
    def getCredentials(self):
        """Gets valid user credentials from storage.

        If nothing has been stored, or if the stored credentials are invalid,
        the OAuth2 flow is completed to obtain the new credentials.

        Returns:
            Credentials, the obtained credential.
        """
        cwd_dir = os.getcwd()
        credential_dir = os.path.join(cwd_dir,
                                     '.credentials')
        if not os.path.exists(credential_dir):
            os.makedirs(credential_dir)
        credential_path = os.path.join(credential_dir,
                                       'google-drive-credentials.json')

        store = Storage(credential_path)
        credentials = store.get()
        if not credentials or credentials.invalid:
            flow = client.flow_from_clientsecrets(self.CLIENT_SECRET_FILE, 
                                                            self.SCOPES)

            flow.user_agent = self.APPLICATION_NAME
            if flags:
                credentials = tools.run_flow(flow, 
                                            store, 
                                            flags)

            else: # Needed only for compatibility with Python 2.6
                credentials = tools.run(flow, 
                                        store)
            print('Storing credentials to ' + credential_path)
        return credentials
    def creds(self):
        creds = None
        # The file token.pickle stores the user's access and refresh tokens, and is
        # created automatically when the authorization flow completes for the first
        # time.
        if os.path.exists('token.pickle'):
            with open('token.pickle', 'rb') as token:
                creds = pickle.load(token)
        # If there are no (valid) credentials available, let the user log in.
        if not creds or not creds.valid:
            if creds and creds.expired and creds.refresh_token:
                creds.refresh(Request())
            else:
                flow = InstalledAppFlow.from_client_secrets_file(
                        'client_secret.json', self.SCOPES)
                creds = flow.run_local_server(port=0)
            # Save the credentials for the next run
            with open('token.pickle', 'wb') as token:
                pickle.dump(creds, token)
        return creds
    


from tkinter import messagebox
import xlwings as xw
from pynvn.excel import convertrangaphatonunber,returnrangelastcolumn
def returnactivewbpath (namefile):
    """ return active workbook """
    try:
        fpath = xw.books.active.fullname
        return fpath
    except:
        messagebox.showerror ("Error",
                                        "Not yet open file excel, open excel file {0}".format(namefile))
        


import re
import ast
from pynvn.list.str import Concatenatelistofstring
def returnseplistintbbystr(strint):
    """ return list int by separate from string
        ex: "BB5:CF100" ---> [5, 100]
     """
    return list(map(int, (re.findall('\d+', strint)))) 

def str_seplistintbbystr(strint):
    """ return list int_str by separate from string
        ex: "BB5:CF100" ---> ['BB5', 'CF100']
    """
    return list(map(str, (re.findall('\w+', strint)))) 
def str_returnliststr (strint):
    """ 
        return list str convert int list 
        ex: '[1,2,3]' ---> [1, 2, 3]
    """
    return ast.literal_eval(strint)
def returnlist_from_listinstr (strint):
    """ 
        return list str convert int list 
        ex: '[1,2,3]' ---> [1, 2, 3]
    """
    return ast.literal_eval(strint)    
def returnliststr_from_str (strint):
    """ 
        return list str from str (only str without int)
        ex: 'AB1001:A2000' ---> [AB,A]
    """
    pattern = '[a-zA-Z]+'
    return re.findall(pattern, strint) 
def str_seplistint_strbystr(strint):
    """ 
    return list int_str by separate from string
    ex: "BB5:CF100" ---> ['BB5', 'CF100']
    """
    return list(map(str, (re.findall('\w+', strint)))) 

def add_sb_to_str(strint = None, specsb = "$"):
    """ return list int_str by separate from string
        ex: "BB5:CF100" ---> $BB$5:$CF$100
    """
    lstr = returnliststr_from_str(strint)
    lint =returnseplistintbbystr(strint)
    return specsb + lstr[0] + specsb + str(lint[0]) + ":" + specsb + lstr[1] + specsb + str(lint[1])  

def __removespace (instr = None, option = "both"):
    """remove space from string"""
    if option == "both":
        return instr.strip()
    if option == "left":
        return instr.lstrip()
    if option == "right":
        return instr.rstrip()
    else:
        messagebox.showerror("Error","No case for this")

def __capitalize(instr = ""):
    return instr.capitalize()


def splitstrtolist(strin = "",pattern = '\s'):
    """"
    split string
    ex: 'The rain in Vietnam.' ----> ['The', 'rain', 'in', 'Vietnam.']
    """
    strin = __removespace(instr=strin,
                            option="both")
    return re.split(pattern, strin) 
def capitalizefs_list (instr = "",pattern='\s'):
    """
    to capital (uppercase) letter.
    
    ex: nguyen van nhuan ---> ['Nguyen', 'Van', 'Nhuan']

    """
    lstr = splitstrtolist(strin=instr,
                            pattern=pattern
                            )
    return list(map(__capitalize,lstr))

def addvaluexk(listf = [],valuetoadd=" "):
    """ add value xen ke """
    liststr = []
    lenc = len(listf)
    for count,ele in enumerate(listf,1):
        if count != lenc:
            liststr.append(ele)
            liststr.append(valuetoadd)
        else:
            liststr.append(ele)
    return liststr


def capitalizefs (instr = "",pattern='\s'):
    """
    to capital (uppercase) letter.
    
    ex: nguyen van nhuan ---> Nguyen Van Nhuan
    """
    lcap = capitalizefs_list(instr = instr,
                            pattern=pattern)
    ladded = addvaluexk(listf=lcap)
    return Concatenatelistofstring(ladded)

def capitalize(instr = "",option = []):
    """capitalize for string """
    upper_all_fun = {
                "fl":lambda: instr.capitalize(),
                "fs": lambda: capitalizefs(instr=instr),
                "upper_all": lambda: instr.upper(),
                "lower_all": lambda: instr.lower(),
                }
    return upper_all_fun[option]()

def capitalizes(instr = "",options = []):
    """capitalizes for string """
    if (instr == "" or instr ==None):
        return instr
    else:
        for option in options:
            instr = capitalize(instr=instr,option=option)
        return instr


from tkinter import messagebox
import os
import shutil
from pynvn.path.ppath import listfileinfolder
def listfolderofpfolder(folderchild):
    """ return list folder of parent folder"""
    os.chdir(folderchild)
    try:
        lfolderp = os.listdir(folderchild)
    except:
        messagebox.showerror ("Error"," No folder parent folder: {}".format(folderchild))
    return lfolderp
def remove_folder(path):
    """ remove folder of path"""
    # check if folder exists
    if os.path.exists(path):
         # remove if exists
         shutil.rmtree(path,ignore_errors=False)
    else:
         # throw your exception to handle this special scenario
         raise XXError("your exception") 


def filterlistbylstr (criteria = [], 
                        liststr = [],
                        criteriamand = [],
                        criteria_is_not = False,
                        upper = False
                        ):
    """ return list filter by criteria is list str and liststr is str """
    if criteria_is_not:
        if upper:
            return [elestr.upper() for elestr in liststr if (not any(ele in elestr for ele in criteria) and all(elem in elestr for elem in criteriamand))]
        else:
            return [elestr for elestr in liststr if (not any(ele in elestr for ele in criteria) and all(elem in elestr for elem in criteriamand))]
    else:
        return [elestr for elestr in liststr if (any(ele in elestr for ele in criteria) and all(elem in elestr for elem in criteriamand))]

def pairlistandlist(listm, list_str):
    """ return pair of list combine list aphabe """
    return [str(ele[0] + ":" + ele[1]) for ele in  [[list_str[0] +str(eler[0] + 1 ),list_str[1] + str(eler[1] - 1)] for eler in list(zip(listm, listm[1:]))]]


def filterlistbylstr1 (criteria = [], liststr = [],criteriamand = [],reverse_criteria = False):
    return [elestr for elestr in liststr if ele not in elestr for ele in criteria]

def filter_lstr (criteria = [], 
                        liststr = [],
                        criteriamand = [],
                        reverse_criteria = False,
                        upper = False
                        ):
    """ return list filter by criteria is list str and liststr is str """
    if reverse_criteria:
        if upper:
            return [elestr.upper() for elestr in liststr if (not any(ele in elestr for ele in criteria) and all(elem in elestr for elem in criteriamand))]
        else:
            return [elestr for elestr in liststr if (not any(ele in elestr for ele in criteria) and all(elem in elestr for elem in criteriamand))]
    else:
        return [elestr for elestr in liststr if (any(ele in elestr for ele in criteria) and all(elem in elestr for elem in criteriamand))]


import csv
from pynvn.data import returnvaluelist
class csvdata:
    def __init__(self,path_Cof = None,
                     NumberRow = 0
                                    ):
        self.path_Cof = path_Cof
        self.NumberRow = NumberRow

    def ReturnDataAllRowByIndexpath (self):
        #open and read excel 
        with open(self.path_Cof) as csvFile:
            readcsv =csv.reader(csvFile,
                             delimiter=',')
            readcsv = list(readcsv)
            RowNumber = readcsv[self.NumberRow]
        csvFile.close()
        RowNumber.pop(0)
        return RowNumber
    # return data is list by row index 
    def returndatalistrowbyindex (self):
        # get data from conf
        arrlist  = self.ReturnDataAllRowByIndexpath()
        # conver list to string 
        cvstringfromlist = (','.join(arrlist))
        #remove all space 
        cvstringfromlist = cvstringfromlist.strip()
        # return csv from list 
        retl = returnvaluelist(cvstringfromlist,",.(:)",",")
        try:
            retl = [int(i) for i in retl]
        except:
            retl = [str(i) for i in retl]
        return retl

# return data lis trowby index
def returndatalistrowbyindex (path_Cof = None,
                                NumberRow = 0
                                            ):
    csvdata1 = csvdata(path_Cof = path_Cof,
                    NumberRow = NumberRow
                                        )
    return csvdata1.returndatalistrowbyindex()
    
#Return Data All Row By Index path
def ReturnDataAllRowByIndexpath (path_Cof = None,
                                NumberRow = 0
                                            ):
    csvdata1 = csvdata(path_Cof = path_Cof,
                    NumberRow = NumberRow
                                        )
    return csvdata1.ReturnDataAllRowByIndexpath()


class dimrec:
    """ create text and arrow"""
    def __init__ (self, cavas = None, 
                    locationtext = [0,0],
                    locationarrow =[100,100], 
                    text = None,
                    angle = 0):

        self.cavas = cavas
        self.locationtext = locationtext
        self.locationarrow = locationarrow
        self.text = text
        self.angle = angle
    def createarrow (self):
        try:

            self.cavas.delete(self.rll ) # remove
        except:
            pass

        self.rll = self.cavas.create_line(*self.locationarrow,
                                                        fill = "red",
                                                        arrow = "both"
                                                        )
    
    def createtext(self):
        """create text """
        try:

                self.cavas.delete(self.cvt ) # remove
        except:
                pass

        self.cvt = self.cavas.create_text(*self.locationtext, 
                                                        anchor="n",
                                                        text =self.text , 
                                                        angle=self.angle
                                                        )


from tkinter import messagebox

class coordp:
    """find center from 2 point"""
    def __init__ (self,topleftp = [0,0], 
                    bottomrightp = [100,100],
                    topleftk = [0,0],
                    bottomrightk = [0,0],
                    rev_point = True,
                    rev_direction = "left",
                    dis_dim = 20,
                    dis_direc = 20,
                    ):
                    self.topleftp = topleftp
                    self.bottomrightp = bottomrightp
                    self.topleftk = topleftk
                    self.bottomrightk = bottomrightk
                    self.rev_point = rev_point
                    self.dis_dim = dis_dim
                    self.rev_direction = rev_direction
                    self.dis_direc = dis_direc

    def pointstartend(self):
        """ dim get point start and end of widget parent"""
        return self.__pointstartendf(self.topleftp,self.bottomrightp)
    def pointstartendk(self):
        """ dim get point start and end of widget kid"""
        return self.__pointstartendf(self.topleftk,self.bottomrightk) 

    def centertowpoint(self):
        """ center two boy"""
        self.pointstartend()
        return  self.centerpoint
    def centertowpointk(self):
        """ center two point of kid"""
        self.pointstartendk()
        return  self.centerpoint
    
    def fronttowpoint(self):
        """ return front 2 point start and end """
        self.spointf = [self.bottomrightp[0] - self.dis_dim, 
                        self.topleftp[1]]
        self.epointf = [self.bottomrightp[0] - self.dis_dim,
                        self.topleftk[1]]
        return [*self.spointf,*self.epointf]

    def fronttowpointcenter(self):
        """ return front center point of start and end"""
        self.fronttowpoint()
        return [self.spointf[0],(self.spointf[1] + self.epointf[1])/2]
    
    def backtowpoint(self):
        """ return front 2 point start and end """
        self.spointb = [self.bottomrightp[0] - self.dis_dim,
                        self.bottomrightp[1]]
        self.epointb = [self.bottomrightp[0] - self.dis_dim,
                        self.bottomrightk[1]]
        return [*self.spointb,
                *self.epointb]

    def backtowpointcenter(self):
        """ return front center point of start and end"""
        self.backtowpoint()
        return [self.spointb[0],
                (self.spointb[1] + self.epointb[1])/2]

    def lefttowpoint(self):
        """ return left 2 point start and end """
        self.spointl = [self.topleftp[0],
                        self.bottomrightp[1]- self.dis_dim]
        self.epointl = [self.topleftk[0],
                        self.bottomrightp[1]- self.dis_dim]
        return [*self.spointl,
                *self.epointl]

    def lefttowpointcenter(self):
        """ return left center point of start and end"""
        self.lefttowpoint()
        return [(self.spointl[0] + self.epointl[0])/2,self.spointl[1]]

    def righttowpoint(self):
        """ return right 2 point start and end """
        self.spointr = [self.bottomrightp[0],
                        self.bottomrightp[1]- self.dis_dim]
        self.epointr = [self.bottomrightk[0],
                        self.bottomrightp[1]- self.dis_dim]
        return [*self.spointr,*self.epointr]

    def righttowpointcenter(self):
        """ return right center point of start and end"""
        self.righttowpoint()
        return [(self.spointr[0] + self.epointr[0])/2,
                self.spointr[1]]
    
    def centerpointkid(self):
        return [(self.topleftk[0] + self.bottomrightk[0])/2,
                (self.topleftk[1] + self.bottomrightk[1])/2]
    
    def frontcenterpointp(self):
        """ return front center point """
        return [(self.topleftp[0] + self.bottomrightp[0])/2,
                self.topleftp[1] - self.dis_direc]

    def backcenterpointp(self):
        """ return back center point """
        return [(self.topleftp[0] + self.bottomrightp[0])/2,
                self.bottomrightp[1] + self.dis_direc]

    def leftcenterpointp(self):
        """ return left center point """
        return [(self.topleftp[0] - self.dis_direc),(self.bottomrightp[1] + self.topleftp[1]) / 2]

    def rightcenterpointp(self):
        """ return right center point """
        return [self.bottomrightp[0] + self.dis_direc,
                (self.bottomrightp[1] + self.topleftp[1])/2]

    def centerpoinparent(self):
        return [(self.topleftp[0] + self.bottomrightp[0])/2,
                (self.topleftp[1] + self.bottomrightp[1])/2]

    @property
    def rev_direction(self):
        return self._rev_direction
    
    @rev_direction.setter
    def rev_direction(self, direction):
        self._rev_direction = direction

    @property
    def dis_dim(self):
        return self._dis_dim
    
    @dis_dim.setter
    def dis_dim(self, dis_dim):
        self._dis_dim = dis_dim

    def __pointstartendf(self,topp, bottomp):
        """ dim get point start and end of widget parent"""
        if self.rev_direction == "left":

            spoint  = [topp[0] - self.dis_dim,
                            topp[1]]

            epoint  = [topp[0] - self.dis_dim,
                            bottomp[1]]

            self.centerpoint = [spoint[0],
                            (spoint[1] + epoint[1])/2]
        
        elif self.rev_direction == "right":
            spoint  = [bottomp[0]+ self.dis_dim,
                            bottomp[1]]

            epoint  = [bottomp[0]+ self.dis_dim,
                            topp[1]]

            self.centerpoint = [spoint[0],
                                spoint[1] + epoint[1]]
        
        elif self.rev_direction == "bottom": 
            spoint  = [bottomp[0],
                            bottomp[1] + self.dis_dim]

            epoint  = [topp[0],
                            bottomp[1] + self.dis_dim]

            self.centerpoint = [spoint[0] + epoint[0] ,
                                spoint[1]]

        elif self.rev_direction == "top": 
            spoint  = [bottomp[0],
                            topp[1] - self.dis_dim]

            epoint  = [topp[0],
                            topp[1] - self.dis_dim] 
            self.centerpoint = [(spoint[0] + epoint[0])/2,
                                spoint[1]]      
        else:

            messagebox.showerror(title="error message", 
                                message="Recheck rev_direction key")

        if self.rev_point:
            spoint_n = spoint 
            epoint_n = epoint 
        else:
            spoint_n = epoint 
            epoint_n = spoint 
        return [*spoint_n,*epoint_n]
    




    


import openpyxl as xl
import xlwings as xw
from tkinter import messagebox,Label
from pynvn.excel.hdata import hexcel_sep
from pynvn.path.ppath import getdirpath,refullpath
from pynvn.csv.rcsv import returndictrowforcsv
from pynvn.excel import col2num,book_by_path

class hexcel_child:
    """copy excel to excel"""
    def __init__(self,
                    pathtocopy= None,
                    namesheetchild = "AZB",
                    pathconf = None,
                    diplaywindow  = None
                ):
        self.diplaywindow = diplaywindow
        self.pathconf = pathconf
        self.pathtocopy = pathtocopy
        self.namesheetchild = namesheetchild
        self.dicrowconf = returndictrowforcsv(path=pathconf)
        self.__namefile=self.dicrowconf["khns_namfile"]
        self.__Getlistsheet()
    def __Getlistsheet(self):
        self.__app = xw.App(visible=True,
                            add_book= False)
        self.__wb1  = self.__app.books.open(self.pathtocopy,
                                            update_links=False)
        #self.__wb1  = xw.Book(self.pathtocopy)
        #self.__wb1  = book_by_path(path=self.pathtocopy, visible = False)
        self.names = self.__wb1.sheets
        self.lsheetname = [sheet.name for sheet in self.names]
        self.__ws1 = self.__wb1.sheets[self.names[0]] 
        self.__wsname = self.__ws1.name
        # set active sheet
        
        #max row ws1
        self.rows = self.__ws1.api.UsedRange.Rows.count
        #max colum ws1
        self.cols = self.__ws1.api.UsedRange.Columns.count
        # check name sheet (have to have AZB)
        if self.namesheetchild  in self.__wsname:
            pass
        else:
            messagebox.showerror("error", 
                                "Name sheet must start \
                                from symbols {}...".format(self.namesheetchild))

        self.__dirpath = getdirpath(self.pathtocopy)

        self.__fpath = refullpath(dirpath=self.__dirpath,
                                        filename=self.__namefile)

        #self.__wbthns  = book_by_path(path=self.__fpath, visible = False)
        #self.__wbthns  = xw.Book(self.__fpath)
        self.__wbthns  = self.__app.books.open(self.__fpath,
                                            read_only=True,
                                            ignore_read_only_recommended=True,
                                            update_links=False
                                            )

        self.__wb1.sheets[self.__wsname].activate()

        if self.__wbthns == None:
            messagebox.showerror ("Error directory", "Directory {0} not exists, \
                                recheck Directory again Note: extension of excel maybe xls or xlsx,\
                                     check file config {1} have parameter 'khns_namfile'".format(self.__fpath,self.pathconf))
        
    def runaz30azb60(self, onlyitemiden = False):
        """ run AZB30 and run AZB60"""
        exelh = hexcel_sep(wsheet=self.__ws1,
                                dpath=self.__dirpath,
                                namefile=self.__namefile,
                                dicrowconf = self.dicrowconf,
                                wbnsct=self.__wbthns,
                                pathconf = self.pathconf
                                )
        print (self.lsheetname[0])
        if self.lsheetname[0] == "AZB-30":
            if onlyitemiden:
                exelh.itemiden()
            exelh.habz30()
        if self.lsheetname[0] == "AZB-60":
            exelh.habz60()
        if self.lsheetname[0] == "AZB-50" :
            exelh.azb50()
        if self.lsheetname[0] == "AZB-40" :
            exelh.azb40()
        try:
            self.__wb1.save()
        except:
            messagebox.showerror("error",
                                    "Check path for Pfile {}".format(self.pathtocopy))
        
        #self.__wbthns.app.quit()
        self.__wbthns.close()
        self.__wb1.close()
        self.__app.quit()


from pynvn.list.flist import filterlistbylstr
def plus2list(list1, list2):
    """ plus 2 list and sort it """
    return sorted(list1 + list2 )

def pairiterationlist (listin):
    """ Pair iteration in list 
    ex1: [0, 1, 2, 3, 4, 5] --> [(0, 1), (1, 2), (2, 3), (3, 4), (4, 5)]
    ex2: ["a", "f","n", 3, 4, 5] --> [('a', 'f'), ('f', 'n'), ('n', 3), (3, 4), (4, 5)]
    """
    return list(zip(listin, listin[1:]))

def pairlistandlist(listm, list_str):
    """ return pair of list combine list aphabe """
    #res1 =  [[list_str[0] +str(eler[0] + 1 ),list_str[1] + str(eler[1] - 1)] for eler in list(zip(listm, listm[1:])) ]
    return [str(ele[0] + ":" + ele[1]) for ele in  [[list_str[0] +str(eler[0] + 1 ),list_str[1] + str(eler[1] - 1)] for eler in list(zip(listm, listm[1:]))]]
def listpairfrom2list(list_a, list_b):
    """
    pair two list in python
    ex: list_a=[1,2,3],list_b = [4,7,8]) ---> [[1, 4], [2, 7], [3, 8]]
    """
    return [[x, y] for x, y in zip(list_a, list_b)]
def converttoint(ele):
    if type (ele) == float:
        return int(ele)
    else:
        return ele
def convertoint_ifisfloat(listth):
    """ 
    convert to int type is float 
    ex :  ["a","b","c",3.0,2.0,1.0] ---> ['a', 'b', 'c', 3, 2, 1]
    
    """
    result = map(converttoint,
                listth) 
    return list(result)



def rerurnsheet(namefile = "vtt",sheetname ="content_video",client = None):
    """ return sheet """
    sh = client.open(namefile)
    sheet = sh.worksheet(sheetname)  # Open the spreadhseet
    return sheet



import csv
from tkinter import messagebox
def convertcsvtolist(path):
    """ convert csv to list """
    with open(path, newline='') as inputfile:
        results = [row[0] for row in csv.reader(inputfile)]
    return results
def convertcsvto1list(path):
    """ convert csv to one list (ex: [1,2,4,53,23]) """
    with open(path, newline='') as inputfile:
        results = list(csv.reader(inputfile))[0]
        if len(results) == 0:
            messagebox.showerror("Error data in csv",
                                "check data in directory path  {0}".format(path))
    return results


import re
import ast
def returnseplistintbbystr(strint):
    """ return list int by separate from string
        ex: "BB5:CF100" ---> [5, 100]
     """
    return list(map(int, (re.findall('\d+', strint)))) 

def str_seplistintbbystr(strint):
    """ return list int_str by separate from string
        ex: "BB5:CF100" ---> ['BB5', 'CF100']
    """
    return list(map(str, (re.findall('\w+', strint)))) 
def str_returnliststr (strint):
    """ 
        return list str convert int list 
        ex: '[1,2,3]' ---> [1, 2, 3]
    """
    return ast.literal_eval(strint)
def returnlist_from_listinstr (strint):
    """ 
        return list str convert int list 
        ex: '[1,2,3]' ---> [1, 2, 3]
    """
    return ast.literal_eval(strint)

def list_from_listinstr (strint):
    """ 
        return list str convert int list 
        ex: '[1,2,3]' ---> [1, 2, 3]
    """
    return ast.literal_eval(strint)

def returnliststr_from_str (strint):
    """ 
        return list str from str (only str without int)
        ex: 'AB1001:A2000' ---> [AB,A]
    """
    pattern = '[a-zA-Z]+'
    return re.findall(pattern, strint) 
def str_seplistint_strbystr(strint):
    """ 
    return list int_str by separate from string
    ex: "BB5:CF100" ---> ['BB5', 'CF100']
    """
    return list(map(str, (re.findall('\w+', strint)))) 

def add_sb_to_str(strint = None, specsb = "$"):
    """ return list int_str by separate from string
        ex: "BB5:CF100" ---> $BB$5:$CF$100
    """
    lstr = returnliststr_from_str(strint)
    lint =returnseplistintbbystr(strint)
    return specsb + lstr[0] + specsb + str(lint[0]) + ":" + specsb + lstr[1] + specsb + str(lint[1])  



from licensing.models import *
from licensing.methods import Key, Helpers
from pynvn.crypt import write_key,load_key,encrypt,decrypt
from pynvn.mc_id import mcwd_id
import tkinter as tk
def authkey(auth = "WyI0MzQ5NyIsIm8rRGJXcjBJNUo3aWYwUk5URUJNaXdZZWdHSlZZbmwxMHFoK2JEQ0ciXQ==",
            product_id = None,
            rsa_pub_key = "<RSAKeyValue><Modulus>n5Q5BtZIlprf+d74p2YmQT1ZnRrCFGqt9JtAzO29/eNbzaM9rFZ5IaD8iIqbc0gVrE2ZFA0tfCtzeAVdV6MlaDaaqexNIZARMBK4dk9AEZb9kAOzdUXZNLv6O3+HyZg6bV75Gj6xFY17YUCefDol5Fyn0Z072lFXUV1DArgb+i2r/YDBI/QTS0crHMUS7iXdlWRk1DdGABvrvtoR78P6+uci5njxjlkniByBODyRMAoml1zk9YBRrCEXi6HLxlurd2Y29QizHRTCACCZP3WsNSiyKZqgKOOjUnZyi+hMX8+W06tcofsjjbKa7D+csFQi0MeL5juiNM3om0vtSD6zjQ==</Modulus><Exponent>AQAB</Exponent></RSAKeyValue>",
            key = None,
            pathtokey = None,
            pathtovaluecsv_key = None,
            using_permanent_key = False,
            valueser_key = "",
            path_mc_id = "",
            ser_key = ""
            ):
    """ auth key """
    result = Key.activate(token=auth,\
                   rsa_pub_key=rsa_pub_key,\
                   product_id=product_id, \
                   key=key,\
                   machine_code=Helpers.GetMachineCode()
                   )
    if (result[0] == None or not Helpers.IsOnRightMachine(result[0])) and using_permanent_key ==  False:
        # an error occurred or the key is invalid or it cannot be activated
        # (eg. the limit of activated devices was achieved)
        # in this case not connect server 
        if "11001" in  result[1] :
            return _active_or_not(pathtokey =pathtokey,
                                            pathtovaluecsv_key = pathtovaluecsv_key 
                                            )
        # case for not actived 
        else:
            write_key(path=pathtokey)
            key = load_key(pathtokey)
            encrypt(filename=pathtovaluecsv_key,
                    key = key,
                    nametow=b"Not actived")
            return [False]

    elif using_permanent_key:
        return  __check_permanent_key(path_mc_id = path_mc_id,
                                        valueser_key=valueser_key,
                                        pathtokey=pathtokey,
                                        ker_ser=key,
                                        ser_key = ser_key
                                        )
            
    else:
        # everything went fine if we are here!
        write_key(path=pathtokey)
        key = load_key(pathtokey)
        encrypt(filename=pathtovaluecsv_key,
                key = key,
                nametow=b"actived")
        license_key = result[0]
        return [True,str(license_key.expires)]
def _active_or_not(pathtokey,pathtovaluecsv_key):
    """ check if it can not connect to server """
    key = load_key(pathtokey)
    k = decrypt(filename=pathtovaluecsv_key,
                key = key
                )
    if k == "actived":
        return [True]
    else:
        return [False]
def __check_permanent_key(path_mc_id,
                            valueser_key,
                            pathtokey,
                            ker_ser,
                            ser_key
                            ):
    key = load_key(ser_key)
    try:
        key_value_de =  decrypt(filename=valueser_key,
                                key = key
                                )
    except:
        key_value_de = ""

    try:
        idmc = decrypt(filename=path_mc_id,
                        key = key
                        )
    except:
        idmc = ""
    if mcwd_id() == idmc  and key_value_de == ker_ser:
        return [True,"The key is used indefinitely"]
    else:
        return [False]


from tkinter import messagebox
import re
import csv
import ast
def sepnumberandstrfromstr (sstr):
    """Splitting text and number in string"""
    # Splitting text and number in string 
    try:
        temp = re.compile("([a-zA-Z]+)([0-9]+)(\D)([a-zA-Z]+)([0-9]+)") 
        return temp.match(sstr).groups() 
    except:
        messagebox.showerror ("error input","Check your input {}, it must aann:aann".format(sstr))
def returnrangewolastrow(sstr):
    """Splitting text and number in string"""
    try:
        temp = re.compile("([a-zA-Z]+)([0-9]+)(\D)([a-zA-Z]+)") 
        return temp.match(sstr).group() 
    except:
        messagebox.showerror ("error input","Check your input {}, it must aann:aann".format(sstr))
def restrnotspaceifhavespace(instr = None,revalue = "None"):
    """ remove space from string"""
    return instr.replace(" ", "")

def no_accent_vietnamese(s):
    """Remove Vietnamese Accents"""
    if isinstance(s,str):
        s = re.sub(r'[àáạảãâầấậẩẫăằắặẳẵ]', 'a', s)
        s = re.sub(r'[ÀÁẠẢÃĂẰẮẶẲẴÂẦẤẬẨẪ]', 'A', s)
        s = re.sub(r'[èéẹẻẽêềếệểễ]', 'e', s)
        s = re.sub(r'[ÈÉẸẺẼÊỀẾỆỂỄ]', 'E', s)
        s = re.sub(r'[òóọỏõôồốộổỗơờớợởỡ]', 'o', s)
        s = re.sub(r'[ÒÓỌỎÕÔỒỐỘỔỖƠỜỚỢỞỠ]', 'O', s)
        s = re.sub(r'[ìíịỉĩ]', 'i', s)
        s = re.sub(r'[ÌÍỊỈĨ]', 'I', s)
        s = re.sub(r'[ùúụủũưừứựửữ]', 'u', s)
        s = re.sub(r'[ƯỪỨỰỬỮÙÚỤỦŨ]', 'U', s)
        s = re.sub(r'[ỳýỵỷỹ]', 'y', s)
        s = re.sub(r'[ỲÝỴỶỸ]', 'Y', s)
        s = re.sub(r'[Đ]', 'D', s)
        s = re.sub(r'[đ]', 'd', s)
    return s
def converlistinstrtolist(listinstr = '["A","B" ,"C" ," D"]', path = None):
    """ convert list in string to list """
    reader = csv.reader(open(path, 'r'))
    return {k:ast.literal_eval(v)  for k,v in reader}
def removespace (instr = None, 
                option = "both"
                ):
    """remove space from string"""
    rspace_fun = {
                  "both": lambda: instr.strip(),
                  "left": lambda: instr.lstrip(),
                  "right": lambda: instr.rstrip(),
                  "tspacetoospace": lambda: twospace_to_onespace(instr=instr),
                  "all": lambda: instr.replace(" ", "")
                }
    return rspace_fun[option]()

def removespaces(instr = "", 
                options = []):

    """
    Remove space from string with list option \n
    instr: input string to function \n
    options: features for user input
    
    """
    if (instr == "" or instr ==None):
        return instr
    else:
        for option in options:
            instr = removespace(instr=instr,
                            option=option)
        return instr
        
def allspaces_to_onespace(instr = None):
    """
    remove multiple spaces in a string to one space 
    ex: "The     quick brown    fox" to 'The quick brown fox'
    """
    return re.sub(' +', ' ', instr)


def allspaces_to_onespace(instr = None):
    """
    remove multiple spaces in a string to one space 
    ex: "The     quick brown    fox" to 'The quick brown fox'
    """
    return re.sub(' +', ' ', instr)


def cformat_str(instr = None):
    """ check format of str """
    instr =  instr.replace(" ", "")
    if re.match("^([a-zA-Z]+)$",instr):
        return "a"
    if re.match("^([0-9]+)$",instr):
        return "n"
    if re.match("(^[a-zA-Z]+)([0-9]+)$",instr):
        return "an"
    if re.match("([a-zA-Z]+)([0-9]+)(\D)([a-zA-Z]+)([0-9]+)$",instr):
        return "anan"
    else:
        messagebox.showerror ("Error", "check format {0}".format(instr))
def remove_all_space (instr = None):
    """ remove all space """
    instr.replace(" ", "")



import tkinter as tk

class AutoScrollbar(tk.Scrollbar): 

    """ Defining set method with all  """
    # its parameter 
    
    def set(self, low, high): 

        if float(low) <= 0.0 and float(high) >= 1.0: 

            # Using grid_remove 

            self.tk.call("grid", "remove", self) 

        else: 

            self.grid() 

        tk.Scrollbar.set(self, low, high) 
class autoscrollbarctn:
    """ auto scroll bar for container"""
    def __init__ (self,fameparent = None, canvas = None):
        self.fameparent = fameparent
        self.canvas = canvas
        self.__add_scroll_bars
        self.__addcommmandscroll
        self.__conf

    def __add_scroll_bars(self):

        """add scroll bar """

        # Defining vertical scrollbar 

        self.verscrollbar = AutoScrollbar(self.fameparent) 

        # Calling grid method with all its 

        # parameter w.r.t vertical scrollbar 

        self.verscrollbar.grid(row=0, column=1,sticky=tk.N+tk.S) 

        # Defining horizontal scrollbar 

        self.horiscrollbar = AutoScrollbar(self.fameparent,orient=tk.HORIZONTAL) 

        # Calling grid method with all its  

        self.horiscrollbar.grid(row=1, column=0, sticky=tk.E+tk.W) 


    def __addcommmandscroll (self):

        """set and add scroll bar """

        self.canvas.config(xscrollcommand=self.horiscrollbar.set, 

                            yscrollcommand=self.verscrollbar.set)

        self.verscrollbar.config(command=self.canvas.yview) 

        self.horiscrollbar.config(command=self.canvas.xview) 


    def __conf (self):

        """ Configuring canvas """  

        #self.framecv.update_idletasks()

        self.canvas.bind("<Configure>",

                            self.onFrameConfigure)                       #bind an event whenever the size of the viewPort frame changes.


    def onFrameConfigure(self, event):                                              

        '''Reset the scroll region to encompass the inner frame'''

        self.canvas.configure(scrollregion=self.canvas.bbox("all"))                 #whenever the size of the frame changes, alter the scroll region respectively.


from pynvn.excel.toexcel import toexcel
from tkinter import messagebox
import openpyxl as xl
import string
from pynvn.stringnvn  import sepnumberandstrfromstr,returnrangewolastrow
import xlwings as xw
from xlwings.constants import DeleteShiftDirection
from pynvn.path.ppath import getdirpath,ExtractFileNameFromPath

def returnsheet (path, namesheet = "TONG HOP HM"):
    """ return sheet name by index and path excel """
    wb1 = xl.load_workbook(filename=path)
    ws1 = wb1[namesheet]
    return ws1
def  repathlinkexcel (dpath = None ,
                    namefile = None,
                    namesheet = None,
                    usingfullname = False, 
                    fullname = None 
                    ):
    """ return path link excel """
    if usingfullname:
        dpath = getdirpath(fullname)
        namefile = ExtractFileNameFromPath(path=fullname)
        pfile = "'" + dpath + "/" + "[" + namefile + "]" + namesheet + "'"
    else:
        pfile = "'" + dpath + "/" + "[" + namefile + "]" + namesheet + "'"
    return pfile

def colnum_string(n):
    """convert colum number become string"""
    string = ""
    while n > 0:
        n, remainder = divmod(n - 1, 26)
        string = chr(65 + remainder) + string
    return string

def relistsheet(path):
    """ return all sheet of excel file """
    wb1 = xl.load_workbook(filename=path)
    names = wb1.sheetnames
    return names
def returnactivesheet(path):
    """ return active sheet """
    wb1 = xl.load_workbook(filename=path)
    sheet = wb1.active
    return sheet
def returnsheetbyname(path = None, sheetname = "PTVT"):
    """return sheet by name """
    wb1 = xl.load_workbook(filename=path)
    return wb1[sheetname]
def mrowandmcolum (path = None):
    "return maxcolumn and max row column"
    wb1 = xl.load_workbook(filename=path)
    wsactive = wb1.active
    return [wsactive.max_row,wsactive.max_column]

def col2num(col):
    """Return number corresponding to excel-style column."""
    num = 0
    for c in col:
        if c in string.ascii_letters:
            num = num * 26 + (ord(c.upper()) - ord('A')) + 1
    return num

def convertrangaphatonunber (rangapha = None ):
    """ convert rang anphab to cell number"""
    a,n,s,a1,n1 =  sepnumberandstrfromstr(sstr=rangapha)
    return  (col2num(a),int(n)),(col2num(a1),int(n1))
def returnrangelastcolumn(stringrang,lrow = 100):
    """ 
        return range excel by range and by last row
        ex: 'A5:A100' ---> 'A5:2000'
        2000 is last row replace in stringrang
    """
    return returnrangewolastrow(sstr=stringrang) + str(lrow)

def delrowbyindexcell (incolumndel = "C", 
                        valueofindexcoldel = None, 
                        wb = None,
                        namesheet = None,
                        startrow =1,
                        endrow = 1000,
                        valuetoendrow = "VTC"
                        ):
    """ delete row by value of cell """
    for i in range (startrow,
                        endrow):
        valuecompare =wb.sheets[namesheet].range(i,
                                                incolumndel ).value 
        k = i
        if (valuecompare == None or valuecompare == ""):
            while True:
                wb.sheets[namesheet].range('{0}:{0}'.format(k)).api.Delete(DeleteShiftDirection.xlShiftUp)
                if (wb.sheets[namesheet].range(k,incolumndel).value != None and (wb.sheets[namesheet].range(k,incolumndel).value != "")) :
                    break
        if wb.sheets[namesheet].range(k,incolumndel).value == valuetoendrow :
            break
def cellcoordbyvalue(max_row = 20, 
                    min_row = 1 , 
                    max_col = 10,
                    min_col= 0 , 
                    sheet = None,
                    valuetofile = None
                    ):
    """find a cell coordinate containing a value """
    if type(min_col) == str:
        min_col = col2num(min_col)
    if type(max_col) == str:
        max_col = col2num(max_col)
    
    for row in range(min_row, max_row + 1):
        for col in range(min_col, max_col + 1):
            if sheet.range((row,col)).value == valuetofile:
                #print("The Row is: "+str(row)+" and the column is "+str(col))
                return [row,col]
                break
    else:
        messagebox.showerror ("Error", "can not find name value {0} at sheet {1} location column index {2} to {3}, check again".format(valuetofile,
                                                                                                                                        sheet.name,
                                                                                                                                        min_col,
                                                                                                                                        max_col))
        raise ValueError("can not find name value")
    return lrowcol
def lcellindexbyvalue(lvalue, 
                        max_row = 20, 
                        min_row = 1 , 
                        max_col = 10,
                        min_col= 0 , 
                        sheet = None,
                    ):
    """ return list cell index row by list value """
    return  [cellcoordbyvalue(max_col=max_col,
                                        max_row=max_row,
                                        min_row=min_row,
                                        min_col=min_col,
                                        sheet=sheet,
                                        valuetofile=value)[0] for value in lvalue
            ]
def openexcelbyxl (pathex):
    """ open excel by pathex"""
    if (pathex != "" and pathex != None):
        if not xw.apps:
            app = xw.App(visible= True,add_book=False)
        else:
            app = xw.apps.active
        wb1  = app.books.open(pathex)
    #xw.Book(pathex)
def valuebyindexrowcell(lindexcell = None,
                        col = None,
                        sheet = None):
    """ return list value with indexcell 
    for indexcell in lindexcell:
        print (indexcell,sheet,"{0}{1}".format(col,indexcell))
        print (sheet.range("{0}{1}".format(col,indexcell)).formula,indexcell)
    """
    return [sheet.range("{0}{1}".format(col,indexcell)).formula for indexcell in lindexcell]
def closeallfileexcel (namek_ofpname):
    """ close all file excel by list if it is openning filter by namek_ofpname """
    listwb = xw.books
    for wb in listwb:
        if namek_ofpname in wb.name:
            res = messagebox.askyesno("Check file is opening","file name {0} is opening, Please close it, do you close and save it ?".format(wb.name))
            if res:
                quit_excel(wb)
            else:
                break
def quit_excel(wb):
    """
    quit workbook 
    wb: workbook object from xlwings
    """
    # look if PERSONAL.XLSB is in the list of books associated with the Excel App
    if "PERSONAL.XLSB" in [b.name for b in wb.app.books]:
        if len(wb.app.books) == 2: 
            wb.app.quit()
        else:
            wb.close()
    else:
        if len(wb.app.books) == 1: 
            wb.app.quit()
        else:
            wb.close()
def rangecopyrefsamesheet (sheet = None,
                            formulasfirstcell = None,
                            col_index = None,
                            startrow = None,
                            endrow = None):
        """copy range have reference at col_index"""

        sheet.range(startrow,col_index).value =   formulasfirstcell 

        vtformulas = sheet.range(startrow,
                                        col_index).formula

        sheet.range("{0}{1}:{0}{2}".format(colnum_string(col_index),
                                            startrow,
                                            endrow)).formula = vtformulas

def returnvaluekeyim (cola,listvalue_im,sheet,indexrow_im):
        """ return value by cell index and list value  """
        for count,numberint in enumerate(indexrow_im,0):
            sheet.range("{0}{1}".format(cola,
                                        numberint)).value = listvalue_im[count]
def activesheet():
    """ return active sheet of excel file """
    try: 
        return xw.sheets.active
    except:
        messagebox.showerror("Error","No excel file has been opened yet, please open the excel file")

def activesheet_name():
    """ return active sheet name """
    return activesheet().name

def activeworkbook(namefile = None,checknamefile = False):
    """ return active workbook """
    try:
        if not xw.apps:
            app = xw.App(visible= True,add_book=False)
        else:
            app = xw.apps.active

        if checknamefile:
            if namefile == xw.books.active.name:
                return app.books.active
            else:
                messagebox.showerror ("error file name","name active workboook is not {0}".format(namefile))
        else:
            return app.books.active
    except:
        messagebox.showerror("Error","No excel file has been opened yet, please open the excel file")

def activeworkbook_fullname(**kw):
    """ return active workbook full name """
    return activeworkbook(**kw).fullname

def listsheet_by_wb(wb):
    """ return all sheet name of wb, input wb"""
    return [sheet.name for sheet in wb.sheets ]


def listsheetofwb (path):
    """ return all sheet name of wb"""
    if (path != "" and path != None):
        if not xw.apps:
            app = xw.App(visible= True,add_book=False)
        else:
            app = xw.apps.active
        wb1  = app.books.open(path)
        return listsheet_by_wb(wb1)

def check_open_wb (path):
    """ check workbook open or not  """
    try:
        listwb = xw.books
        for wb in listwb:
            if wb.name in path:
                return True
                break
        else:
            return False
    except:
        return False

def sheet_by_namesheet (path, namesheet,**kw):
    """ return sheet name of wb"""
    if (path != "" and path != None):
        if not xw.apps:
            app = xw.App(add_book=False,**kw)
        else:
            app = xw.apps.active
        wb1  = app.books.open(path)
        ws = wb1.sheets[namesheet]
        if ws == None:
            messagebox.showerror("Error", "Check sheet name input:  {0}".format(str(namesheet).upper()))
        return ws
    else:
        messagebox.showerror("Error", "No workbook is opened or Check sheet name input:  {0}".format(str(namesheet).upper()))
def book_by_path (path,**kw):
    """ return sheet name of wb"""
    if (path != "" and path != None):
        app = xw.App(add_book=False,**kw)
        wb1  = app.books.open(path)
        return wb1
def ws_by_namesheet (path = None, namesheet = "Can not find sheet",cre_new_app = True,**kw):
    """ return sheet name of ws"""
    wb1 = open_wb_by_xl(pathex=path,**kw)
    ws = wb1.sheets[namesheet]
    if ws == None:
        messagebox.showerror("Error", "Check sheet name input:  {0}".format(str(namesheet).upper()))
    return ws
def open_wb_byxl (pathex):
    """ open excel by pathex"""
    if (pathex != "" and pathex != None):
        if not xw.apps:
            app = xw.App(visible= True,add_book=False)
        else:
            app = xw.apps.active
        return app.books.open(pathex)

def open_wb_by_xl (pathex,cre_new_app = True,**kw):
    """ open excel by pathex"""
    if (pathex != "" and pathex != None):
        if cre_new_app:
            app = xw.App(**kw)
        else:
            app = xw.App(**kw) if not xw.apps else  xw.apps.active
        return app.books.open(pathex)
    else:
        messagebox.showerror("Error", "Check directory path input")


from openpyxl import Workbook
import openpyxl
import xlwings as xw 
from pynvn.excel import mrowandmcolum,sheet_by_namesheet
from pynvn.excel import (convertrangaphatonunber,
                            returnrangelastcolumn)
class credict:
    """ create dict for value and key """
    def __init__ (self, ws = None,
                        rangea = "C7:C1000", 
                        pathfull = None, 
                        namesheet ="PTVT1", 
                        engine ="openpyxl"):
        self.rangea = rangea
        self.engine = engine
        self.ws = sheet_by_namesheet(path=pathfull,
                                        namesheet=namesheet)

        self.lastrow = self.ws.range('A' + str(self.ws.cells.last_cell.row)).end('up').row

        self.rangea = returnrangelastcolumn(stringrang=rangea,lrow=self.lastrow)
    def reindexrownotnone(self):
        """ renturn index which value not none"""
        key_list = [cell.row for cell in self.ws.range(self.rangea) if cell.value != None]
        return key_list

    def revaluerownotnone(self):
        """ renturn value which value not none"""

        value_list = [cell.value for cell in self.ws.range(self.rangea) if cell.value != None]

        return value_list
    def redictvaluesandvaluecol(self, columnumber = 4, removeemtyvalue = True):
        """ return dict value and value column"""
        rnn = self.reindexrownotnone()
        rvm = self.revaluerownotnone()
        arrch = []
        res = list(zip(rnn, rnn[1:] + rnn[:1])) 
        for eler in res:
            arrchild = self.__listchild(eler,columnumber=columnumber)
            arrch.append(arrchild)
            arrchild = None
        dictionary = dict(zip(rvm, arrch))

        if removeemtyvalue:
            new_dict = {key:val for key, val in dictionary.items() if len(val) != 0} 
        else:
            new_dict = dictionary
        return new_dict
    
    def returndictvaluebyindexcolumnandrow(self, 
                                            value_criteria_range, 
                                            range_col ="D7:D1000", 
                                            indexcolumn = [5,6,8]):
        """ return dict by value rangce_col and indexcolumn """
        indexrcevalu = [[self.valuebycol_row(cell.row,indexcolumn[0]),
                                self.valuebycol_row(cell.row,indexcolumn[1]),
                                self.valuebycol_row(cell.row,indexcolumn[2])] \
                                for cell in self.ws.range(range_col) if cell.value == value_criteria_range]

        return indexrcevalu
            
    def valuebycol_row(self,irow,icolumn):
        """ return value cell by index column and row"""
        valuebycolr = self.ws.range(irow,icolumn).value 
        return valuebycolr

    def __listchild(self,ele,columnumber = 4):
        s,t = ele
        arrchild = [[self.ws.range((ie,columnumber)).row,
                        self.ws.range((ie,columnumber)).value] for ie in range(s,t) if self.ws.range((ie,
                                                                                                    columnumber)).value != None] 
        return arrchild

    def listothercell (self,irow,icolumn):
        """ return value of column sheet ptvl1"""
        valuebycolr = self.ws.range(irow,icolumn).value 
        return valuebycolr



from pynvn.excel import col2num,colnum_string, repathlinkexcel
from pynvn.csv.todict import returndictrowforcsv
from pynvn.excel.fomulas import vlookup,sumif,anyfun
from pynvn.excel import (sheet_by_namesheet,
                        activeworkbook_fullname,
                        activesheet_name,
                        activesheet)
import xlwings as xw
from tkinter import messagebox
class fformulas:
    """ fill the formulas into excel file """
    def __init__(self, retr_path = None,
                        retr_sheetname =None, 
                        des_path = None,
                        des_sheetname =None,
                        fuction = None,
                        pathconf = None,
                        ):
        self.dictconf = returndictrowforcsv(path=pathconf)
        self.__retr_sheetname = retr_sheetname
        self.__des_path = des_path
        self.__des_sheetname = des_sheetname
        self.__fuction = fuction

        if retr_sheetname == "Active Sheet":
            retr_path = activeworkbook_fullname()
            retr_sheetname = activesheet_name()

        self.__pexcelretr = repathlinkexcel(usingfullname=True,
                                            fullname=retr_path,
                                            namesheet=retr_sheetname
                                            )
        if des_sheetname == "Active Sheet":
            self.__ws_des = activesheet()
        else:
            self.__ws_des = sheet_by_namesheet(path=des_path,
                                                namesheet=des_sheetname)
    def filltoexcell(self):
        if self.__fuction == "VLOOKUP":
            vlvalue = self.dictconf["sub_vlookup_lookup_value"]
            vTaalue = self.dictconf["sub_vlookup_table_array"]
            vrlalue = self.dictconf["sub_vlookup_range_lookup"]
            clgvalue = self.dictconf["sub_vlookup_column_get_value"]
            clrvvalue = self.dictconf["sub_vlookup_loc"]
            vlookup(loopkup_value_range=vlvalue,
                                table_array=vTaalue,
                                plexcel=self.__pexcelretr,
                                colum_to_get_value = clgvalue,
                                ws_des = self.__ws_des,
                                Sub_VLOOKUP_Locc_result_value= clrvvalue
                                ).forexelldes()
        elif self.__fuction == "SUMIF":
            sirange = self.dictconf["sub_sumif_range"]
            sicriteria = self.dictconf["sub_sumif_criteria"]
            sisum_range = self.dictconf["sub_sumif_sum_range"]
            silrvalue = self.dictconf["sub_sumif_loc"]
            sumif(sirange=sirange,
                    sicriteria=sicriteria,
                    sisum_range=sisum_range,
                    plexcel=self.__pexcelretr,
                    ws_des=self.__ws_des,
                    silrvalue=silrvalue
                    ).forexelldes()
        else:
            any_range = self.dictconf[self.__fuction.lower()]

            any_loc = "sub_" + self.__fuction.lower() + "_loc"
            loc_value = self.dictconf[any_loc]

            anyfun(function_range=any_range,
                    function_loc=loc_value,
                    plexcel=self.__pexcelretr,
                    ws_des=self.__ws_des
                    ).forexelldes()


from pynvn.check.list import check_list_value
from xlwings.constants import DeleteShiftDirection
from pynvn.excel import colnum_string

def delrowbyrange (incolumndel = 5, 
                        ws = None,
                        startrow =1,
                        endrow = 1000,
                        value_to_end = "VTC",
                        valuetodelete = ["",None],
                        using_value_to_end = True
                        ):
    """ 
    Delete row by value of cell \n
    incolumndel: index of column to delete \n
    ws: worksheet execute \n
    startrow: start row in ws (worksheet) to start  \n
    endrow: row in ws (worksheet) to end  \n
    value_to_end: value to End for loop \n
    valuetodelete: delete row if valule in this \n
    using_value_to_end: option from progamer \n

    """
    for i in range (startrow,endrow):
        valuecompare =ws.range(i,incolumndel ).value 
        if check_list_value(valuetocheck =[valuecompare],
                            not_in_checkvalue= False):
            while True:
                ws.range('{0}:{0}'.format(i)).api.Delete(DeleteShiftDirection.xlShiftUp)
                if check_list_value(valuetocheck=[ws.range(i,incolumndel).value]):
                    break
        if using_value_to_end:
            if ws.range(i,incolumndel).value == value_to_end :
                break
        else:
            lr = ws.range(colnum_string(incolumndel) + str(ws.cells.last_cell.row)).end('up').row
            if lr == i:
                break

def del_row_by_valueinrange (index_col = 5, 
                            ws = None,
                            startrow =1,
                            endrow = 1000,
                            value_to_end = "VTC",
                            valuetodelete = ["",None],
                            using_value_to_end = True
                            ):

    """
    Delete row by value of cell \n
    index_col: index of column to delete \n
    ws: worksheet execute \n
    startrow: start row in ws (worksheet) to start \n
    endrow: row in ws (worksheet) to end  \n
    value_to_end: value to End for loop \n
    valuetodelete: delete row if valule in this \n
    using_value_to_end: option from progamer \n
    """
    for i in range (startrow,endrow):
        valuecompare =ws.range(i,index_col ).value 
        if check_list_value(valuetocheck =[valuecompare],
                            not_in_checkvalue= False):
            while True:
                ws.range('{0}:{0}'.format(i)).api.Delete(DeleteShiftDirection.xlShiftUp)
                if check_list_value(valuetocheck=[ws.range(i,index_col).value]):
                    break
        if using_value_to_end:
            if ws.range(i,index_col).value == value_to_end :
                break
        else:
            lr = ws.range(colnum_string(index_col) + str(ws.cells.last_cell.row)).end('up').row
            if lr == i:
                break


from tkinter import messagebox
from pynvn.path.ppath import (
                            resource_path_is_from_pyinstall_and_dev,
                            ExtractFileNameFromPath,
                            IsRunningInPyinstallerBundle,
                            PathFromFileNameAndDirpath,
                            check_pfile,
                            refullpath,
                            getpathfromtk,
                            ask_open,
                            mfileopen
                            )
from pynvn.path.pcsv import (ReturnDataAllRowByIndexpath,
                            returndatalistrowbyindex,
                            )


class bidirectional_iterator(object):
    def __init__(self, collection):
        self.collection = collection
        self.index = 0
        self.k = 0 
        self.n = 0

    def next(self):
        if self.n != 0:
            self.index =  self.index + 1
            self.n = 0
        result = self.collection[self.index]
        self.index = self.index + 1
        self.k += 1
        if self.index > len(self.collection)- 1:
            self.index = 0
        return result

    def prev(self):
        self.n += 1
        if self.k != 0:
            self.index =  self.index - 2
            self.k = 0
        else:
            self.index =  self.index - 1
        if self.index < 0:
            self.index = len(self.collection)- 1
        return self.collection[self.index]

    def __iter__(self):
        return self


class towlist:
    """handling 2 list """
    def __init__ (self, list1= None, list2 = None):
        self.list1 = list1
        self.list2 = list2

    def subtracttowlist(self):
        """sub tract 2 list"""
        difference = []
        #initialization of result list
        zip_object = zip(self.list1, 
                        self.list2)
        for list1_i, list2_i in zip_object:
            difference.append(list1_i-list2_i)
        return difference

    def plustracttowlist(self):
        """plus tract 2 list"""
        difference = []
        #initialization of result list
        zip_object = zip(self.list1, 
                        self.list2)
        for list1_i, list2_i in zip_object:
            difference.append(list1_i+list2_i)
        return difference




import os
from tkinter import Entry

class namefile:
    def  __init__(self, dirpath = None,
                        fnamesub = None
                ):
        self.dirpath = dirpath
        self.fnamesub = fnamesub
    def returnallfilenameinfolder(self):
        entries = os.listdir(self.dirpath)
        return entries
    def returnfilfullnamefromsubname(self):
        entries = self.returnallfilenameinfolder()
        for etry in entries:
            if self.fnamesub in etry:
                return etry
        else:
            print ("can not find file")
            
        


from pynvn.caculate.area import area
class writetext:
    def __init__ (self,canvas =None,
                topleftkid = None,
                toprightkid = None,
                centerpoint = [0,0]
                ):
        self.canvas = canvas
        self.centerpoint = centerpoint
        self.topleftkid = topleftkid
        self.toprightkid = toprightkid
        

    def warea(self,**kwargs):
        """ write area """

        are_k = area(topleftpoint= self.topleftkid, 
                                bottomrightpoint=self.toprightkid).areafromtopbottompoint()

        # create text 
        try:
            self.canvas.delete(self.tca ) # remove
        except:
            pass
        #coordrcenter = self.coord.centerpointkid()

        self.tca = self.canvas.create_text(*self.centerpoint, 
                                                        anchor="center",
                                                        text ="Area to build: {}".format(are_k), 
                                                        angle=0,
                                                        **kwargs)
        


from pynvn.stringnvn.slist import str_seplistint_strbystr,returnliststr_from_str,add_sb_to_str
from pynvn.excel import col2num,colnum_string


def formula_ex(name_function = "vlookup"):
    def wrap(f):
        def wrapped_f(*args):
            if name_function == "vlookup":
                ws_des,loopkup_value_range,table_array,\
                rang_lookup, plexcel,colum_to_get_value,\
                Locc_result_value,lockrange = args
                if lockrange:
                    table_array = add_sb_to_str(table_array)
                cellstart = str_seplistint_strbystr(loopkup_value_range)[0]
                re_value = returnliststr_from_str(table_array)[0]
                cell_start_locvalue = str_seplistint_strbystr(Locc_result_value)[0]
                # index column number
                col_index_num = col2num(colum_to_get_value) -  col2num(re_value)  + 1

                fomularex = "=IFERROR(VLOOKUP({2},{1}!{0},{3},{4}),{5})".format(table_array,
                                                                                plexcel,
                                                                                cellstart,
                                                                                col_index_num,
                                                                                rang_lookup,
                                                                                '"' + "" + '"'
                                                                                )               

            if name_function == "sumif":

            
            f(ws_des,cell_start_locvalue,fomularex,Locc_result_value)
        return wrapped_f
    return wrap



class vlookup:
    """using function vlookup"""
    def __init__(self,
                 ws_des = None,
                 loopkup_value_range = None,
                 table_array = None, 
                 rang_lookup = False, 
                 plexcel = None,
                 colum_to_get_value = None,
                 Sub_VLOOKUP_Locc_result_value = None,
                 lockrange = True
                 ):

        self.__ws_des = ws_des
        self.__table_array = table_array
        if lockrange:
            self.__table_array = add_sb_to_str(self.__table_array)
        self.__plexcel = plexcel
        self.__rang_lookup = rang_lookup
        self.__loopkup_value_range = loopkup_value_range
        self.__Sub_VLOOKUP_Locc_result_value = Sub_VLOOKUP_Locc_result_value
        self.__cellstart = str_seplistint_strbystr(loopkup_value_range)[0]
        re_value = returnliststr_from_str(table_array)[0]
        self.__cell_start_locvalue = str_seplistint_strbystr(Sub_VLOOKUP_Locc_result_value)[0]
        
        # index column number
        self.__col_index_num = col2num(colum_to_get_value) -  col2num(re_value)  + 1

    def valueformulas(self):

        fomularex = "=IFERROR(VLOOKUP({2},{1}!{0},{3},{4}),{5})".format(self.__table_array,
                                                                    self.__plexcel,
                                                                    self.__cellstart,
                                                                    self.__col_index_num,
                                                                    self.__rang_lookup,
                                                                    '"' + "" + '"'
                                                                    )
        return fomularex

    def for_excel_des(self):
        self.__ws_des.range(self.__cell_start_locvalue).value = self.valueformulas()
        vtformulas = self.__ws_des.range(self.__cell_start_locvalue).formula
        self.__ws_des.range(self.__Sub_VLOOKUP_Locc_result_value).formula = vtformulas

class sumif:
    """using function sumif"""
    def __init__(self,sirange = None,
                    sicriteria = None, 
                    sisum_range = None, 
                    plexcel = None,
                    ws_des = None,
                    silrvalue = None,
                    lockrange = True
                    ):
        self.__sirange = sirange
        self.__sicriteria = sicriteria
        self.__sisum_range = sisum_range
        self.__silrvalue = silrvalue
        self.__ws_des = ws_des
        self.__plexcel = plexcel
        self.__cell_start_locvalue = str_seplistint_strbystr(silrvalue)[0]

        if lockrange:
            self.__sirange = add_sb_to_str(self.__sirange)
            self.__sisum_range = add_sb_to_str(self.__sisum_range)

    def valueformulas(self):

        fomularex = "=SUMIF({0}!{1},{2},{0}!{3})".format(self.__plexcel, self.__sirange,self.__sicriteria,self.__sisum_range)

        return fomularex
    def for_excel_des(self):
        self.__ws_des.range(self.__cell_start_locvalue).value = self.valueformulas()
        vtformulas = self.__ws_des.range(self.__cell_start_locvalue).formula
        self.__ws_des.range(self.__silrvalue).formula = vtformulas


class anyfun:
    """using function for any function"""

    def __init__(self,function_range = None,
                    function_loc = None,
                    plexcel = None,
                    ws_des = None
                    ):
        self.__function_range = function_range
        self.__function_loc = function_loc
        self.__ws_des = ws_des
        self.__plexcel = plexcel
        self.__cell_start_locvalue = str_seplistint_strbystr(self.__function_loc)[0]

    def for_excel_des(self):
        self.__ws_des.range(self.__cell_start_locvalue).value = self.__function_range
        vtformulas = self.__ws_des.range(self.__cell_start_locvalue).formula
        self.__ws_des.range(self.__function_loc).formula = vtformulas


from pynvn.list import listpairfrom2list,convertoint_ifisfloat
from pynvn.stringnvn.slist import returnliststr_from_str
import string
def listbyrangeremoveduplicate(sheetexcel,rangea):
    """ return list excel remove duplicate"""
    return list(set(sheetexcel.range(rangea).value))

def listbyrange(sheetexcel,rangea,removeduplicate = False):
    """ return list excel by range"""
    if removeduplicate:
        return list(set(sheetexcel.range(rangea).value))
    else:
        return sheetexcel.range(rangea).value

def pairslistfromexcel (startrow= 1, 
                        floc = "A", 
                        sloc = "B",
                        convetfloattointat_slot = True,
                        sheet = None,
                        ):
    """ create pair list from floc and sloc of excel """
    # max row at floc 
    m_rowatfloc = sheet.range(floc + str(sheet.cells.last_cell.row)).end('up').row
    # create list from range at floc 
    listfloc = sheet.range("{0}{1}:{0}{2}".format(floc,startrow,m_rowatfloc)).value
    # create list from range at sloc 
    listsloc = sheet.range("{0}{1}:{0}{2}".format(sloc,startrow,m_rowatfloc)).value
    if convetfloattointat_slot:
        listsloc = convertoint_ifisfloat(listsloc)
        
    return listpairfrom2list(list_a=listfloc,
                            list_b=listsloc)

def remove2side_pair(lpair):
    """ 
        remove 2 side space list pair ele
        ex: ["  abc",1] ---> ["abc",1]
    """

    return [remove2side(lpair[0]),remove2side(lpair[1])]

def remove2side(in_para):
    """ 
    remove 2 side keep int, remove str 
    ex: "  kkk"----> "kkk"
            12 ---> 12
    """

    if (type(in_para) == int or type(in_para) == float):
        return in_para
    else:
        return str(in_para).strip()
    
def remove_2side_space_byindex_plist(l_instr = [],side = "both"):
    """ remove 2 side space by index pair list """
    if side == "both":
        return list(map(remove2side_pair, l_instr))

def removevalueinlistpair(lista,
                        deleteifvalue = [None,""],
                        lower_index_0 = True,
                        remove_2side_space_index_0_1 = True
                        ):
    """
    remove value in list pair by list deleteifvalue
    """
    if lower_index_0:
        listpair = [[pairarr[0].lower(),pairarr[1]] for pairarr in lista if pairarr[0] not in deleteifvalue]
    if remove_2side_space_index_0_1:
        listpair = remove_2side_space_byindex_plist(l_instr=listpair)
    else:
        listpair = [pairarr for pairarr in lista if pairarr[0] not in deleteifvalue]
    return listpair

def lnumbercolumnbyrangstr (rstr = None):
    """ return range number by rstr by excel column """
    lstr = returnliststr_from_str(strint=rstr)
    if len(lstr) == 2:
        a,b = lstr
        return [inte for inte in range(_col2num(a),_col2num(b) +1)]
    elif len(lstr) == 1:
        return [_col2num(lstr[0])]

def lacolumnbyrangstr (rstr = []):
    """ return range string by rstr by excel column """
    lint = lnumbercolumnbyrangstr(rstr=rstr)
    return list(map(_colnum_string,lint))

def _col2num(col):
    """Return number corresponding to excel-style column."""
    num = 0
    for c in col:
        if c in string.ascii_letters:
            num = num * 26 + (ord(c.upper()) - ord('A')) + 1
    return num

def _colnum_string(n):
    """conver colum number become string"""
    string = ""
    while n > 0:
        n, remainder = divmod(n - 1, 26)
        string = chr(65 + remainder) + string
    return string


def modify_listpair(lista,
                    deleteifvalue = [None,""],
                    lower_index_0 = True,
                    remove_all_space_index_1 = True
                    ):
    """
    remove value in list pair by list deleteifvalue
    """

    if (lower_index_0 == True and remove_all_space_index_1 == True):

        listpair = [[pairarr[0].lower(),pairarr[1].replace(" ", "")] for pairarr in lista if pairarr[0] not in deleteifvalue]

    elif (lower_index_0 == True and remove_all_space_index_1 == False) :

        listpair = [[pairarr[0].lower(),pairarr[1]] for pairarr in lista if pairarr[0] not in deleteifvalue]

    else:

        listpair = [pairarr for pairarr in lista if pairarr[0] not in deleteifvalue]

    return listpair


from pynvn.list.twolist import towlist
class placereccenter:
    """ place center window cavas"""
    def __init__ (self,info_width_P = 100,
                    info_height_p = 100,
                    info_width_k = 100,
                    info_height_k = 100,
                    startpoint = [0,0]):

                self.startpoint = startpoint
                self.info_width_P = info_width_P
                self.info_height_p = info_height_p
                self.info_width_k = info_width_k
                self.info_height_k = info_height_k


    def pointleftrec (self):
        """ return point rectangle on the left"""
        return towlist(self.pointcenterofparent(),
                                    [self.info_width_k/2, 
                                    self.info_height_k/2
                                    ]
                                    ).subtracttowlist()

    def pointrightrec(self):
        """ return point rectangle on the left""" 
        return towlist(self.pointcenterofparent(),
                                    [self.info_width_k/2, 
                                    self.info_height_k/2
                                    ]
                                    ).plustracttowlist()

    def pointcenterofparent (self):
        """ point center of window parent"""
        return towlist(self.startpoint,
                                    [self.info_width_P/2,
                                    self.info_height_p/2
                                    ]
                                    ).plustracttowlist()


class setbackdimention:
    """set back dimention from rectangle"""
    def __init__ (self,w_front = 100,
                    w_back = 100,
                    w_left = 100,
                    w_right = 100,
                    topleftpoint_p = [0,0],
                    bottomrightpoint_p = [100,100]):
                self.w_front = w_front
                self.w_back = w_back
                self.w_left = w_left
                self.w_right = w_right
                self.topleftpoint_p = topleftpoint_p
                self.bottomrightpoint_p = bottomrightpoint_p
    
    def topleftpoint(self):
        """ return top left of window kid"""
        return towlist (list1= self.topleftpoint_p,
                list2= [self.w_left,self.w_front]).plustracttowlist()

    def toprightpoint(self):
        """ return top right of window kid"""
        return towlist (list1= self.bottomrightpoint_p,
                        list2= [self.w_right,self.w_back]).subtracttowlist()

class create_poly_from_tleft_bright:
    """create polygon from top lelf and bottom right """
    def __init__ (self,topleftpoint_p = [0,0],
                    bottomrightpoint_p = [100,100],
                    w_front_r = 100,
                    w_back_r = 100,
                    w_left_r = 100,
                    w_right_r = 100,
                    dis_r = 200
                    ):
                    self.topleftpoint_p = topleftpoint_p
                    self.bottomrightpoint_p = bottomrightpoint_p
                    self.w_front_r = w_front_r
                    self.w_back_r = w_back_r
                    self.w_left_r = w_left_r
                    self.w_right_r = w_right_r
                    self.dis_r = dis_r
    
    def roadfront (self):
        """ create section road front"""
        self.xy1f = [self.topleftpoint_p[0],
                self.topleftpoint_p[1] - self.dis_r
                ]
        self.xyaf= cresectionpoint(point1=self.xy1f,width= self.w_front_r)
        
        self.xy2f = [self.xy1f[0],
                self.xy1f[1] -self.w_front_r
                ]

        self.xy3f = [self.bottomrightpoint_p[0], 
                self.xy2f[1]
                ]
        self.xybf = cresectionpoint(point1=self.xy3f,width= self.w_front_r,direction="y_right")

        self.xy4f = [self.xy3f[0],
                self.xy1f[1]
                ]
        return [*self.xy1f,*self.xyaf,*self.xy2f,*self.xy3f,*self.xybf,*self.xy4f]

    def roadback(self):
        """ create section road back"""
        self.xy1b = [self.bottomrightpoint_p[0],
                self.bottomrightpoint_p[1] + self.dis_r]

        self.xyab = cresectionpoint(point1=self.xy1b,width= self.w_back_r,direction="y_right")

        self.xy2b = [self.xy1b[0],self.xy1b[1] + self.w_back_r]


        self.xy3b = [self.topleftpoint_p[0], self.xy2b[1]]

        self.xybb = cresectionpoint(point1=self.xy3b,width= self.w_back_r)

        self.xy4b = [self.xy3b[0],self.xy1b[1]]
        return [*self.xy1b,*self.xyab,*self.xy2b,*self.xy3b,*self.xybb,*self.xy4b]        
    
    def roadleft(self):
        """ create section road left"""
        self.xy1l = [self.topleftpoint_p[0] - self.dis_r,
                self.topleftpoint_p[1]]
            
        self.xyal = cresectionpoint(point1=self.xy1l,width= self.w_left_r,direction="y_up")

        self.xy2l = [self.xy1l[0] - self.w_left_r,self.xy1l[1] ]

        self.xy3l = [self.xy2l[0], self.bottomrightpoint_p[1]]

        self.xybl = cresectionpoint(point1=self.xy3l,width= self.w_left_r,direction="y_bottom")        

        self.xy4l = [self.xy1l[0],self.xy3l[1]]

        return [*self.xy1l,*self.xyal,*self.xy2l,*self.xy3l,*self.xybl,*self.xy4l]

    def roadright(self):
        """ create section road right"""
        self.xy1r = [self.bottomrightpoint_p[0] + self.dis_r,
                self.bottomrightpoint_p[1]]

        self.xyar = cresectionpoint(point1=self.xy1r,
                            width= self.w_right_r,
                            direction="y_bottom")

        self.xy2r = [self.xy1r[0] + self.w_right_r,self.xy1r[1] ]

        self.xy3r = [self.xy2r[0], self.topleftpoint_p[1]]

        self.xybr = cresectionpoint(point1=self.xy3r,
                            width= self.w_right_r,
                            direction="y_up") 

        self.xy4r = [self.xy1r[0],self.xy3r[1]]

        return [*self.xy1r,*self.xyar,*self.xy2r,*self.xy3r,*self.xybr,*self.xy4r]

    def toprandbottoml_roadfront (self):
        """return top left and bottom right of road front"""
        return [self.xy1f,self.xy3f]
        
    def toprandbottoml_roadback (self):
        """return top left and bottom right of road back"""
        return [self.xy1b,self.xy3b]

    def toprandbottoml_roadleft (self):
        """return top left and bottom right of road left"""
        return [self.xy1l,self.xy3l]

    def toprandbottoml_roadright (self):
        """return top left and bottom right of road left"""
        return [self.xy1r,self.xy3r]

        

def cresectionpoint(point1, width = 100, direction = "y_left",hsh = 6, hsv = 8 ):

    if direction == "y_left":
        p_center = [point1[0],point1[1] + -(width/2)]
        p_t = [p_center[0],p_center[1] - width/hsv ]
        p_d =  [p_center[0],p_center[1] + width/hsv ]
        p_2 = [p_d[0] - width/hsh,p_d[1]]
        p_3 = [p_t[0] + width/hsh,p_t[1]]
        return [*p_d,*p_2,*p_3,*p_t]
    elif direction == "y_right":
        p_center = [point1[0],point1[1] + (width/2)]
        p_t = [p_center[0],p_center[1] - width/hsv ]
        p_d =  [p_center[0],p_center[1] + width/hsv ]
        p_2 = [p_t[0] + width/hsh,p_t[1]]
        p_3 = [p_d[0] - width/hsh,p_d[1]]
        return [*p_t,*p_2,*p_3,*p_d]
    
    elif direction == "y_up":
        p_center = [point1[0]- (width/2),point1[1]]
        p_t = [p_center[0]- width/hsv,p_center[1]]
        p_d =  [p_center[0] + width/hsv,p_center[1]]
        p_2 = [p_d[0] ,p_d[1]+ width/hsh]
        p_3 = [p_t[0] ,p_t[1]- width/hsh]
        return [*p_d,*p_2,*p_3,*p_t]

    elif direction == "y_bottom":
        p_center = [point1[0] + (width/2),point1[1]]
        p_t = [p_center[0]+ width/hsv,p_center[1]]
        p_d =  [p_center[0] - width/hsv,p_center[1]]
        p_2 = [p_d[0] ,p_d[1]+ width/hsh]
        p_3 = [p_t[0] ,p_t[1]- width/hsh]
        return [*p_d,*p_2,*p_3,*p_t]
        
        

    


    





    
    

    


    



from tkinter import messagebox
import openpyxl as xl 
class listexcel:
    """ return list excel or list sheet """
    def __init__ (self, 
                    l_ex = None, 
                    index = 0):

        self.l_ex = l_ex
        self.index = index
    def returnlsheet(self):
        """ return list sheet name  follow list sheet """
        wsl = []
        for els in self.l_ex:
            try:
                elop = xl.load_workbook(els)
                sname = elop.sheetnames[0]
                wsl.append(sname)
            except:
                messagebox.showerror("Error","check file name excel: {0} close it if it is opening".format(els))
        return wsl
